/*
 * Created on 2005. 6. 7
 */
package info.jinwoomin.nevernaver;

import info.jinwoomin.downloader.DownloaderUtils;
import info.jinwoomin.downloader.IDownloadable;
import info.jinwoomin.downloader.IDownloader;
import info.jinwoomin.downloader.html.IHtmlDownloadable;
import info.jinwoomin.downloader.html.IHtmlDownloader;
import info.jinwoomin.downloader.html.IHtmlLineHandler;
import info.jinwoomin.log.AbstractLogabble;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author <a href="mailto:yeoupooh@gmail.com">Jinwoo, Min </a>
 */
public class NaverBlogPage extends AbstractLogabble implements IDownloadable,
        IHtmlDownloadable, IHtmlLineHandler
{
    private ResourceBundle bundle;
    private String outputFolder = "";
    private String postTime = "";
    private String title = "";
    private String categoryName = "";
    private String categoryId = "";
    private String pageId = "";
    private String userId = "";
    private int category;
    private int pageIndex;
    private IDownloader downloader;
    private IHtmlDownloader htmlDownloader;
    final static int STATE_BEFORE = 0;
    final static int STATE_CONTENT = 1;
    final static int STATE_AFTER = 2;
    private int state;
    private StringBuffer lineBuffer;

    /**
     * @param bundle
     *            The bundle to set.
     */
    public void setBundle(ResourceBundle bundle)
    {
        this.bundle = bundle;
    }

    /**
     * @param outputFolder
     *            The outputFolder to set.
     */
    public void setOutputFolder(String outputFolder)
    {
        this.outputFolder = outputFolder;
    }

    /**
     * @param category
     *            The category to set.
     */
    public void setCategory(int category)
    {
        this.category = category;
    }

    /**
     * @param userId
     *            The userId to set.
     */
    public void setUserId(String userId)
    {
        this.userId = userId;
    }

    /**
     * @param pageIndex
     *            The pageIndex to set.
     */
    public void setPageIndex(int pageIndex)
    {
        this.pageIndex = pageIndex;
    }

    public void backup()
    {
        // http://blog.naver.com/post/postList.jsp?blogId=yeoupooh
        URL url;

        // init line buffer
        lineBuffer = new StringBuffer();

        try
        {
            // init html downloader
            String urlString = bundle.getString("base.url");
            urlString = urlString.replace("::USER_ID::", userId);
            urlString = urlString.replace("::CATEGORY_NO::", new Integer(category).toString());
            urlString = urlString.replace("::PAGE_NO::", new Integer(pageIndex).toString());
//            url = new URL("http://blog.naver.com/post/postList.jsp?blogId="
//                    + userId + "&categoryNo=" + category + "&cpage="
//                    + pageIndex);
            url = new URL(urlString);
            htmlDownloader.setLogger(this.logger);
            htmlDownloader.setUrl(url);

            // init binary downloader
            downloader.setLogger(this.logger);
            downloader.removeAllFiles();

            logger.info("page.url=[" + url + "]");

            // download html
            state = STATE_BEFORE;
            htmlDownloader.download();

            // mkdir output folder
            File file = new File(outputFolder + "/" + pageId);
            file.mkdir();

            // download image
            downloader.download();

        }
        catch (MalformedURLException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }

        logger.debug("linebuffer=[" + lineBuffer.toString() + "]");

        // write metainfo
        Properties props = new Properties();
        props.setProperty("categoryId", categoryId);
        props.setProperty("categoryName", categoryName);
        props.setProperty("pageId", pageId);
        props.setProperty("title", title);
        props.setProperty("postTime", postTime);
        try
        {
            //            props.store(new FileOutputStream(outputFolder + "/" + pageId
            //                    + "/metainfo.properties"), "");
            props.storeToXML(new FileOutputStream(outputFolder + "/" + pageId
                    + "/metainfo.xml"), "generated by NeverNaver", "UTF-8");
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }

        // write contents
        try
        {
            String htmlFile = outputFolder + "/" + pageId + "/contents.html";
            logger.info("html.file=[" + htmlFile + "]");
            FileOutputStream fos = new FileOutputStream(htmlFile);
            PrintWriter pw = new PrintWriter(fos);
            pw.println(lineBuffer.toString());
            pw.close();
        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see info.jinwoomin.downloader.IDownloadable#setDownloader(info.jinwoomin.downloader.IDownloader)
     */
    public void setDownloader(IDownloader downloader)
    {
        this.downloader = downloader;
    }

    /*
     * (non-Javadoc)
     * 
     * @see info.jinwoomin.downloader.html.IHtmlDownloadable#setHtmlDownloader(info.jinwoomin.downloader.html.HtmlDownloader)
     */
    public void setHtmlDownloader(IHtmlDownloader htmlDownloader)
    {
        this.htmlDownloader = htmlDownloader;
    }

    private String match(String pattern, String src)
    {
        Pattern p = Pattern.compile(pattern);
        Matcher m = p.matcher(src);
        while (m.find())
        {
            logger.info("m.group(1)=" + m.group(1));
            return m.group(1);
        }

        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see info.jinwoomin.downloader.html.IHtmlLineHandler#handle(java.lang.String)
     */
    public boolean handle(String line)
    {
        Pattern p;
        Matcher m;
        String matchResult;

        if (state == STATE_BEFORE)
        {
            // get category id
            matchResult = match(bundle.getString("pattern.categoryid"), line);
            if (matchResult != null)
            {
                categoryId = matchResult;
                logger.info("categoryId=" + categoryId);
            }

            // get category name
            matchResult = match(bundle.getString("pattern.categoryname"), line);
            if (matchResult != null)
            {
                categoryName = matchResult;
                logger.info("categoryName=" + categoryName);
            }

            // get page id
            matchResult = match(bundle.getString("pattern.pageid"), line);
            if (matchResult != null)
            {
                pageId = matchResult;
                logger.info("pageId=" + pageId);
            }

            // get title
            matchResult = match(bundle.getString("pattern.title"), line);
            if (matchResult != null)
            {
                title = matchResult;
                logger.info("title=" + title);
            }

            // get post time
            matchResult = match(bundle.getString("pattern.posttime"), line);
            if (matchResult != null)
            {
                postTime = matchResult;
                logger.info("posttime=" + postTime);
            }

            if (line
            //					.equals(" <td style='padding: 15 10 35 10' colspan='2'
                    // width='572' class='view'>")) {
                    .indexOf(bundle.getString("start.line")) > -1)
            {
                state = STATE_CONTENT;
            }
        }
        else
            if (state == STATE_CONTENT)
            {
                //			if (line.equals(" </td>")) {
                if (line.indexOf(bundle.getString("end.line")) > -1)
                {
                    state = STATE_AFTER;
                }
                else
                {
                    /*
                     * " <IMG id=[\\w]* style=\"CURSOR: hand\"
                     * onclick='popview(\"http://blogfiles.naver.net/data2/2004/3/7/92/banner-01-01.jpg\")'
                     * src=\"http://blogfiles.naver.net/data2/2004/3/7/92/banner-01-01.jpg\"
                     * onload='setTimeout(\"resizeImage(329089)\",200)'>"
                     */
                    //				String[] patternStrings = {
                    //						"<IMG id=[\\x00-\\xff]* style=\"[\\x00-\\xff]*\"
                    // onclick=[']?popview\\([\"]?[\\x00-\\xff]*[\"]?\\)[']?[
                    // height=[0-9]* | ]+src=\"([\\x00-\\xff]*)\"[ width=[0-9]*
                    // |
                    // ]+onload='setTimeout\\(\"resizeImage\\([0-9]*\\)\",[0-9]*\\)'[
                    // align=[a-z]*]*>",
                    //						"<IMG height=[0-9]* src=\"([\\x00-\\xff]*)\" width=[0-9]*
                    // border=[0-9]*>" };
                    int numOfPattern = new Integer(bundle
                            .getString("pattern.num")).intValue();

                    String resultLine = new String(line);
                    for (int i = 1; i <= numOfPattern; i++)
                    {
                        p = Pattern.compile(bundle.getString("pattern." + i));
                        m = p.matcher(line);
                        while (m.find())
                        {
                            logger.debug("patter=["
                                    + bundle.getString("pattern." + i) + "]");
                            logger.debug("found=[" + m.group(1) + "]");

                            URL url;
                            try
                            {
                                url = new URL(m.group(1));
                                downloader.addFile(url, outputFolder
                                        + "/"
                                        + pageId
                                        + "/"
                                        + DownloaderUtils.pickFilename(m
                                                .group(1)));
                            }
                            catch (MalformedURLException e)
                            {
                                e.printStackTrace();
                            }
                            resultLine = resultLine.replace(m.group(1),
                                    DownloaderUtils.pickFilename(m.group(1)));
                        }
                    }
                    lineBuffer.append(resultLine + "\n");
                }
            }
        return false;
    }
}
